#include "mex.h"
#include "libwtf.h"

#include <string>
#include <opencv2/opencv.hpp>
#include <cmath>

using namespace cv;
using namespace std;
using namespace wtf;

vector<double> mxArray2vector(const mxArray* mxArr) {
    double* mxPr = (double*)mxGetPr(mxArr);
    int size = mxGetM(mxArr) * mxGetN(mxArr);
    vector<double> rtn(mxPr, mxPr + size);
    return rtn;
}

void mexFunction (
    int nlhs, mxArray* plhs[],      /*Output variables */
    int nrhs, const mxArray* prhs[] /*Input variables */
    )
{
    if (nlhs != 2)
    {
        mexErrMsgTxt("Output error. (# should be = 2)");
        return;
    }
    if (nrhs != 17)
    {
        mexErrMsgTxt("Input error");
        return;
    }
    
    vector<double>  homography;     /* pointers to homography matrix */
    double  w1, h1, w2, h2;
    string  histogram_shape;
    int     n_histogram_bins;
    vector<double>  x1s, y1s, a1s, b1s, c1s;
    vector<double>  x2s, y2s, a2s, b2s, c2s;

    float   *v1, *v2;
    
    /* Inputs */
    homography  = mxArray2vector(prhs[0]);
    w1  = (double)mxGetScalar(prhs[1]);
    h1  = (double)mxGetScalar(prhs[2]);
    w2  = (double)mxGetScalar(prhs[3]);
    h2  = (double)mxGetScalar(prhs[4]);
    histogram_shape = mxArrayToString(prhs[5]);
    n_histogram_bins   = (int)mxGetScalar(prhs[6]);
    x1s = mxArray2vector(prhs[7]);
    y1s = mxArray2vector(prhs[8]);
    a1s = mxArray2vector(prhs[9]);
    b1s = mxArray2vector(prhs[10]);
    c1s = mxArray2vector(prhs[11]);
    x2s = mxArray2vector(prhs[12]);
    y2s = mxArray2vector(prhs[13]);
    a2s = mxArray2vector(prhs[14]);
    b2s = mxArray2vector(prhs[15]);
    c2s = mxArray2vector(prhs[16]);
    
    /* Output */
    plhs[0] = mxCreateNumericMatrix(n_histogram_bins+1,1,mxSINGLE_CLASS,mxREAL);
    v1      = (float*)mxGetPr(plhs[0]);
    plhs[1] = mxCreateNumericMatrix(n_histogram_bins+1,1,mxSINGLE_CLASS,mxREAL);
    v2      = (float*)mxGetPr(plhs[1]);
       
    // Convert from config string to HistogramShape
    HistogramShape histogram_shape_;
    if (histogram_shape.compare("dist_2border")==0) {
        histogram_shape_ = DIST2BORDER;
    } else if (histogram_shape.compare("dist_2imgcent")==0) {
        histogram_shape_ = DIST2CENTER;
    }

    // Temp pair of vectors to store the feature vector pair
    pair<vector<float>,vector<float> > v_pair;

    extract_feature_vector(
        homography,
        w1,h1,w2,h2,
        histogram_shape_,n_histogram_bins,
        x1s,y1s,a1s,b1s,c1s,
        x2s,y2s,a2s,b2s,c2s,
        v_pair );

    // Copy the feature vectors to output
    copy(v_pair.first.begin(), v_pair.first.end(), v1);
    copy(v_pair.second.begin(), v_pair.second.end(), v2);

    
    return;
}

