#ifndef EXTRACT_FEATURE_VECTOR_H
#define EXTRACT_FEATURE_VECTOR_H

#include "def.h"

#include <opencv/cv.h>

namespace wtf {

bool extract_feature_vector(
    const cv::Mat &homography,
    const double width1,
    const double height1,
    const double width2,
    const double height2,
    const HistogramShape histogram_shape,
    const int n_histogram_bins,
    const std::vector<double> &xs,
    const std::vector<double> &ys,
    const std::vector<double> &ss,
    std::vector<float> &features
);
std::vector<float> histogram_dist2center(
    const cv::Mat &overlap,
    const cv::Mat &inliers,
    const int n_histogram_bins 
);
std::vector<float> histogram_dist2border(
    const cv::Mat &overlap,
    const cv::Mat &inliers,
    const int n_histogram_bins 
);

bool inlier_map(
    const double width,
    const double height,
    const std::vector<double> &xs,
    const std::vector<double> &ys,
    const std::vector<double> &ss,
    cv::Mat &output 
);
bool overlap_map(
    const cv::Mat &homography,
    const int width1, 
    const int height1,
    const int width2, 
    const int height2,
    cv::Mat &output 
);

}  // namespace wtf

#endif
