#include "train.h"
#include "util.h"

using namespace cv;
using namespace std;

namespace wtf {

/*******************************
 * Public functions in libwtf.h
 *******************************/
CvBoost* train (
    vector<vector<float> > v,
    vector<bool> l,
    const int n_itr )
{
    Mat v_mat = two_D_vector2mat(v);
    vector<int> l_int(l.begin(),l.end());   // Cast bool to int
    Mat l_mat(l_int);
    
    CvBoost* model = train(v_mat,l_mat,n_itr);
    return model;
}
/* END of public functions in libwtf.h 
 **************************************/


/**************************
 * Functions in train.h 
 **************************/
CvBoost* train ( 
    const Mat& v,
    const Mat& l,
    const int n_itr ) 
{
    CvBoost* model = new CvBoost();
    CvBoostParams params = CvBoostParams();	// Default parameters have boost_type = CvBoost::REAL
    // http://docs.opencv.org/modules/ml/doc/boosting.html#cvboostparams-cvboostparams
    params.weak_count = n_itr;
    model->train(v,CV_ROW_SAMPLE,l,Mat(),Mat(),Mat(),Mat(),params,false);
    // http://docs.opencv.org/modules/ml/doc/boosting.html#cvboost-train

    return model;
}
CvBoost* train ( 
    float * v,
    int * l,
    const int d,    // dimension
    const int n,    // # samples
    const int n_itr ) 
{
    Mat v_mat = Mat(n,d,CV_32F,v);
    Mat l_mat = Mat(n,1,CV_32S,l);
    
    CvBoost* model = train(v_mat,l_mat,n_itr);
    return model;
}

}  // namespace wtf


