#include "util.h"

using namespace std;
using namespace cv;

namespace wtf {

Mat two_D_vector2mat(vector<vector<float> > &v) {
    Mat mat(v.size(),v[0].size(),CV_32F);
    for(int i=0; i<mat.rows; ++i)
         for(int j=0; j<mat.cols; ++j)
              mat.at<float>(i,j) = v[i][j];
    return mat.clone();
}

Mat vec2homography(
    vector<double> vec // vec might be changed, therefore pass-by-value
) {
//	cout << "vec.size() = " <<vec.size() << endl;
    if (vec.size()==8) {
        vec.push_back(1);    // Add the last element 1.
    } else if (vec.size()==9) {
        if (vec[8]!=1) {
            cout << "Last element of homography should always be 1" << endl;
            throw "Error";
        }
    } else {
        cout << "Homography has wrong number of elements: " << vec.size() << endl;
        cout << "Should be either 8 or 9." << endl;
        throw "Error";
    }

    Mat homography_mat(vec,false);
    
    return homography_mat.reshape(0,3).clone(); // Make a complete copy
}

double eclipse_scale(double a, double b, double c) {
    double det = sqrt( (a-c)*(a-c) + 4 * b * b );
    double e1 = (a+c+det)*0.5;
    double e2 = (a+c-det)*0.5;
    double l1 = 1.0 / sqrt(e1);
    double l2 = 1.0 / sqrt(e2);

    return sqrt( l1*l2 );
}

}  // namespace wtf